// adapted from
// http://java.sun.com/products/jfc/tsc/articles/threads/threads2.html

import javax.swing.SwingUtilities;

public abstract class InterruptableSwingWorker {

    /** 
     * Class to maintain reference to current worker thread
     * under separate synchronization control.
     */
    private static class ThreadVar {
      private Thread thread;
      ThreadVar(Thread t) { 
        thread = t; 
      }
      synchronized Thread get() { return thread; }
      synchronized void clear() { thread = null; }
    }

    private ThreadVar threadVar;

    public abstract void timeconsumingFunction();  // the time consuming method

    /**
     * Called on Swings event dispatching thread (not on the worker thread)
     * after the <code>timeconsumingFunction</code> method has returned.
     */
    public void finished() {
    }

    public boolean interruptAndWait() {
      // sends the timeconsumingFunction() function the interrupt 
      // and waits until timeconsumingFunction() terminates.
      // returns true if tread was still alive and needed interruption
      // or false else
      Thread t = threadVar.get();
      if (t != null) {
        t.interrupt();   // set the interrupt flag -- assume it is honored fast,
                         // as the following join() is running in Swings event loop
        try {
          t.join();      // wait for timeconsumingFunction() to finish
        }
        catch (InterruptedException e) {
          System.out.println("SwingWorker.join.interrupt() -- should not happen") ;
        }
        threadVar.clear();
        return true ;
      }
      return false ;
    }


    /**
     * create a Thread, that once start()ed, will call timeconsumingFunction(),
     * and when timeconsumingFunction() terminates without having asked by interruptAndWait(), 
     * will call finished() in Swings event loop
     */
    public InterruptableSwingWorker() {
      final Runnable doFinished = new Runnable() {
        public void run() { finished(); }
      };

      Runnable doTimeconsumingFunction = new Runnable() { 
        public void run() {
          try {
            timeconsumingFunction() ;
          }
          finally {
            threadVar.clear();
          }
          if (!Thread.currentThread().isInterrupted())
            SwingUtilities.invokeLater(doFinished);
        }
      };

      threadVar = new ThreadVar(new Thread(doTimeconsumingFunction));
    }

    public void start() {
      Thread t = threadVar.get();
      if (t != null) 
        t.start();
    }
}

